--[[

A line graph display showing historic data. It is only meant as an example of a very 
different display system, and it is highly inefficient.

Does not add anything to Skada's memory usage permanently; only keeps a local cache
of the last 30 entries (of whatever is being shown) for each player.

Shortcoming: only a subset of legends are shown, depending on window size, and there is no way to scroll.

--]]

local L = LibStub("AceLocale-3.0"):GetLocale("Skada", false)
local Skada = Skada

local mod = Skada:NewModule("HistoricLineDisplay")
local libwindow = LibStub("LibWindow-1.1")
local graph = LibStub("LibGraph-2.0")
local media = LibStub("LibSharedMedia-3.0")

-- Add to Skada's enormous list of display providers.
mod.name = "Historic line display"
Skada.displays["line"] = mod

-- Called when a Skada window starts using this display provider.
function mod:Create(window)
	-- Skada "windows" are only logical windows. We need to create a frame.
	if not window.frame then
		-- Our local cache of historic data.
		window.historycache = {}
		
		-- Our cache of colors, which persists on mode changes etc.
		window.colorcache = {}
		
		-- X value for next entry in history.
		window.num = 0
		
		window.frame = CreateFrame("Frame", window.db.name.."LineChartFrame", UIParent)
		window.frame.win = window
		window.frame:SetMovable(true)
		window.frame:SetClampedToScreen(true)
		window.frame:SetWidth(window.db.width or 300)
		window.frame:SetHeight(window.db.height or 180)
		window.frame:SetBackdrop( {bgFile = media:Fetch("background", "Solid"), tile = false, edgeSize = 0, insets = {left = 0, right = 0, top = 0, bottom = 0} } )
		window.frame:SetBackdropColor(0, 0, 1, 0.2)
		window.frame:SetPoint("TOPLEFT", UIParent, "CENTER", 0, 0)
		window.frame:EnableMouse()
		window.frame:SetScript("OnMousedown", function(f, button) if button == "RightButton" then window:RightClick() end end)
	
		window.frame.legendframe = CreateFrame("Frame", nil, window.frame)
		window.frame.legendframe:SetWidth(140)
		window.frame.legendframe:SetPoint("TOPRIGHT", window.frame, "TOPRIGHT", 0, 0)
		window.frame.legendframe:SetPoint("BOTTOMRIGHT", window.frame, "BOTTOMRIGHT", 0, 0)
		window.frame.legendframe.AddLegend = Skada.AddLegend
		window.frame.legendframe.ClearLegends = Skada.ClearLegends
		window.frame.legendframe.ClickLegend = function(id, label, button) Skada.ClickRow(window.frame, window, id, label, button) end
		window.frame.legendframe.EnterLegend = function(id, label) Skada.EnterRow(window.frame, window, id, label) end
		window.frame.legendframe.LeaveLegend = function(id, label) Skada.LeaveRow(window.frame, window, id, label) end
		window.frame.legendframe.AdjustLegends = Skada.AdjustLegends
		Skada.SetupLegendScrolling(window.frame.legendframe)
		
		window.frame.MakeResizable = Skada.MakeResizable
		window.frame.OnResizingInProgress = function(self) window.chart:SetWidth(window.frame:GetWidth()-145); window.chart:SetHeight(window.frame:GetHeight()-2); window:UpdateDisplay(); window.frame.legendframe:AdjustLegends() end
		window.frame:MakeResizable()
		window.frame:SetMinResize(140,40)

		-- Add a header and some snazzy buttons.
		window.header = CreateFrame("Button", nil, window.frame)
		window.header:SetHeight(20)
		window.header:SetPoint("BOTTOMLEFT", window.frame, "TOPLEFT", 0, 0)
		window.header:SetPoint("BOTTOMRIGHT", window.frame, "TOPRIGHT", 0, 0)
		window.header:SetScript("OnMouseDown", function() window.frame:StartMoving() end)
		window.header:SetScript("OnMouseUp", function() window.frame:StopMovingOrSizing(); libwindow.SavePosition(window.frame) end)
		window.header:SetBackdrop( {bgFile = media:Fetch("statusbar", "Aluminium"), tile = false, edgeSize = 0, insets = {left = 0, right = 0, top = 0, bottom = 0} } )
		window.header:SetBackdropColor(0.1, 0.1, 0.3, 0.8)
		window.header:SetText(window.db.name)
		window.header:GetFontString():SetPoint("LEFT", window.header, "LEFT", 5, 1)
		window.header:GetFontString():SetJustifyH("LEFT")
		local myfont = CreateFont("MyTitleFont")
		myfont:SetFont(media:Fetch('font', "Accidental Presidency"), 12)
		window.header:SetNormalFontObject(myfont)
		
		window.header.AddButton = Skada.AddButton
		window.header.AdjustButtons = Skada.AdjustButtons
		
		-- Add window buttons.
		window.header:AddButton(L["Configure"], "Interface\\Addons\\Skada\\images\\icon-config", "Interface\\Addons\\Skada\\images\\icon-config", function() Skada:OpenMenu(window) end)
		window.header:AddButton(L["Reset"], "Interface\\Addons\\Skada\\images\\icon-reset", "Interface\\Addons\\Skada\\images\\icon-reset", function() StaticPopup_Show("ResetSkadaDialog") end)
		window.header:AddButton(L["Segment"], "Interface\\Buttons\\UI-GuildButton-PublicNote-Up", "Interface\\Buttons\\UI-GuildButton-PublicNote-Up", function() Skada:SegmentMenu(window) end)
		window.header:AddButton(L["Mode"], "Interface\\Buttons\\UI-GuildButton-PublicNote-Up", "Interface\\Buttons\\UI-GuildButton-PublicNote-Up", function() Skada:ModeMenu(window) end)
		window.header:AddButton(L["Report"], "Interface\\Buttons\\UI-GuildButton-MOTD-Up", "Interface\\Buttons\\UI-GuildButton-MOTD-Up", function() Skada:OpenReportWindow(window) end)
		
		-- A bunch of legends.
		
		-- Add the actual graph.
		local g=graph:CreateGraphLine("TestLineGraph",window.frame,"TOPLEFT","TOPLEFT",2,0,window.frame:GetWidth()-145,window.frame:GetHeight()-2)
		g:SetGridColor({0,0,0,0.0})
		g:SetYLabels(true,false)
		g:SetAxisDrawing(true,true)
		g:SetAxisColor({1.0,1.0,1.0,1.0})
		g:SetAutoScale(false)
		window.chart = g
		
		--DEFAULT_CHAT_FRAME:AddMessage("Testing Line Graph")
	end
	window.frame:Show()
	
	-- Register with LibWindow-1.0.
	libwindow.RegisterConfig(window.frame, window.db)
	
	-- Restore window position.
	libwindow.RestorePosition(window.frame)
end

-- Called by Skada windows when the window is to be destroyed/cleared.
function mod:Destroy(win)
	wipe(win.colorcache)
	win.colorcache = nil
	win.frame.legendframe:ClearLegends()
	win.frame:Hide()
	win.frame = nil
	win.chart = nil
	win.header = nil
	win.historycache = nil
	win.num = nil
end

-- Called by Skada windows when the window is to be completely cleared and prepared for new data.
function mod:Wipe(win)
	win.chart:ResetData()
	wipe(win.historycache)
	win.num = 0
	win.frame.legendframe:ClearLegends()
end

-- Called by Skada windows when title of window should change.
function mod:SetTitle(win, title)
	-- Set title.
	win.header:SetText(title)
end

-- Called by Skada windows when the display should be updated to match the dataset.
-- We keep a table of historic entries and redraw the entire frame every update. Sayonara fps, hello fpm!
function mod:Update(win)
	-- Some modes may alter title continously.
	win.header:SetText(win.metadata.title)
	for i, data in ipairs(win.dataset) do
		if data.id then
			-- Add this row to our history.
			if win.historycache[data.id] then
				tinsert(win.historycache[data.id].values, {x = win.num, y = data.value})
				
				-- Trim history
				if #win.historycache[data.id] > 30 then
					table.remove(win.historycache[data.id], 1)
				end
			else
				local color
				if win.colorcache[data.id] then
					color = win.colorcache[data.id]
				else
					color = { math.random(), math.random(), math.random() }
					win.colorcache[data.id] = color
				end
				
				win.historycache[data.id] = {num = win.num, values = {x = win.num, y = data.value}, color = color}
				
				-- Legend
				win.frame.legendframe:AddLegend(data.id, data.label, color)
			end
			
		end
	end
	win.frame.legendframe:AdjustLegends()
	
	win.chart:ResetData()
	local min_x, max_x = 0, 0
	local min_y, max_y = 0, 0
	local count = 0
	local cache = {}
	for id, data in pairs(win.historycache) do
		count = count + 1
		if count > 5 then
			break
		end
	
		-- Lots and lots of unnecessary copying going on here.
		local points = {}
		for i, value in ipairs(data.values) do
			tinsert(points, {value.x, value.y})
			
			if value.y < min_y then
				min_y = value.y
			end
			if value.y > max_y then
				max_y = value.y
			end
			if max_x < value.x then
				max_x = value.x
			end
			if value.x < min_x then
				min_x = value.x
			end
		end
		
		tinsert(cache, {p = points, c = data.color})
	end
	win.chart.NeedsUpdate = true
	win.chart:SetXAxis(max(0, min_x), max(1, max_x))
	win.chart:SetYAxis(max(0, min_y), max(1, max_y))
	win.chart:SetGridSpacing(math.floor(max_x / 4), math.floor(max_y / 4))
	for i, stuff in ipairs(cache) do
		win.chart:AddDataSeries(stuff.p, stuff.c)
	end
	win.chart:OnUpdate()
	
	win.num = win.num + 1
end

function mod:Show(win)
	win.frame:Show()
end

function mod:Hide(win)
	win.frame:Hide()
end

function mod:IsShown(win)
	return win.frame:IsShown()
end

-- Called by Skada windows when window settings have changed.
function mod:ApplySettings(win)
	if win.db.barslocked then
		win.header:EnableMouse(false)
		win.frame.resizebutton:Hide()
	else
		win.header:EnableMouse(true)
		win.frame.resizebutton:Show()
	end
end

function mod:AddDisplayOptions(win, options)

end

